@@ -25,6 +25,8 @@ extern int verbose;
 extern int always_checksum;
 extern off_t total_size;
 
+extern int cvs_exclude;
+
 extern int make_backups;
 extern int preserve_links;
 extern int preserve_perms;
@@ -33,22 +35,19 @@ extern int preserve_uid;
 extern int preserve_gid;
 extern int preserve_times;
 
+static char **local_exclude_list = NULL;
 
 /*
   This function is used to check if a file should be included/excluded
   from the list of files based on its name and type etc
-
-  This will be expanded soon to include globbing functions, .ignore
-  files and anything else Paul can dream up :-)
  */
 static int match_file_name(char *fname,struct stat *st)
 {
-
+  if (check_exclude(fname,local_exclude_list)) 
+    return 0;
   return 1;
 }
 
-
-
 static void send_directory(int f,struct file_list *flist,char *dir);
 
 static char *flist_dir = NULL;
@@ -171,8 +170,10 @@ static void send_file_name(int f,struct file_list *flist,
 
   send_file_entry(file,f);
 
-  if (S_ISDIR(file->mode) && recurse) {      
+  if (S_ISDIR(file->mode) && recurse) {
+    char **last_exclude_list = local_exclude_list;
     send_directory(f,flist,file->name);
+    local_exclude_list = last_exclude_list;
     return;
   }
 }
@@ -199,7 +200,12 @@ static void send_directory(int f,struct file_list *flist,char *dir)
   if (fname[l-1] != '/')
     strcat(fname,"/");
   p = fname + strlen(fname);
-  
+
+  if (cvs_exclude) {
+    strcpy(p,".cvsignore");
+    local_exclude_list = make_exclude_list(fname,NULL);
+  }  
+
   for (di=readdir(d); di; di=readdir(d)) {
     if (strcmp(di->d_name,".")==0 ||
 	strcmp(di->d_name,"..")==0)
