@@ -35,12 +35,12 @@ int preserve_uid = 0;
 int preserve_gid = 0;
 int preserve_times = 0;
 int update_only = 0;
+int cvs_exclude = 0;
 
 static int server = 0;
 static int sender = 0;
 static int recurse = 0;
 
-
 static void report(int f)
 {
   int in,out,tsize;
@@ -73,13 +73,62 @@ static void report(int f)
 }
 
 
+static void server_options(char **args,int *argc)
+{
+  int ac = *argc;
+  static char argstr[50];
+  static char bsize[30];
+  int i, x = 1;
+
+  args[ac++] = "--server";
+
+  if (!sender)
+    args[ac++] = "--sender";
+
+  argstr[0] = '-';
+  for (i=0;i<verbose;i++)
+    argstr[x++] = 'v';
+  if (make_backups)
+    argstr[x++] = 'b';
+  if (update_only)
+    argstr[x++] = 'u';
+  if (preserve_links)
+    argstr[x++] = 'l';
+  if (preserve_uid)
+    argstr[x++] = 'o';
+  if (preserve_gid)
+    argstr[x++] = 'g';
+  if (preserve_devices)
+    argstr[x++] = 'D';
+  if (preserve_times)
+    argstr[x++] = 't';
+  if (preserve_perms)
+    argstr[x++] = 'p';
+  if (recurse)
+    argstr[x++] = 'r';
+  if (always_checksum)
+    argstr[x++] = 'c';
+  if (cvs_exclude)
+    argstr[x++] = 'C';
+  argstr[x] = 0;
+
+  args[ac++] = argstr;
+
+  if (block_size != BLOCK_SIZE) {
+    sprintf(bsize,"-B%d",block_size);
+    args[ac++] = bsize;
+  }    
+
+  *argc = ac;
+}
+
+
+
 int do_cmd(char *cmd,char *machine,char *user,char *path,int *f_in,int *f_out)
 {
   char *args[100];
-  int i,x,argc=0;
+  int i,argc=0;
   char *tok,*p;
-  char argstr[30]="-s";
-  char bsize[30];
 
   if (!cmd)
     cmd = getenv(RSYNC_RSH_ENV);
@@ -101,40 +150,8 @@ int do_cmd(char *cmd,char *machine,char *user,char *path,int *f_in,int *f_out)
 
   args[argc++] = RSYNC_NAME;
 
-  x = 2;
-  for (i=0;i<verbose;i++)
-    argstr[x++] = 'v';
-  if (!sender)
-    argstr[x++] = 'S';
-  if (make_backups)
-    argstr[x++] = 'b';
-  if (update_only)
-    argstr[x++] = 'u';
-  if (preserve_links)
-    argstr[x++] = 'l';
-  if (preserve_uid)
-    argstr[x++] = 'o';
-  if (preserve_gid)
-    argstr[x++] = 'g';
-  if (preserve_devices)
-    argstr[x++] = 'D';
-  if (preserve_times)
-    argstr[x++] = 't';
-  if (preserve_perms)
-    argstr[x++] = 'p';
-  if (recurse)
-    argstr[x++] = 'r';
-  if (always_checksum)
-    argstr[x++] = 'c';
-  argstr[x] = 0;
+  server_options(args,&argc);
 
-  args[argc++] = argstr;
-
-  if (block_size != BLOCK_SIZE) {
-    sprintf(bsize,"-B%d",block_size);
-    args[argc++] = bsize;
-  }    
-  
   if (path && *path) {
     char *dir = strdup(path);
     p = strrchr(dir,'/');
@@ -292,34 +309,72 @@ void do_server_recv(int argc,char *argv[])
 }
 
 
-void usage(void)
+static void usage(FILE *f)
 {
-  fprintf(stderr,"rsync version %s Copyright Andrew Tridgell and Paul Mackerras\n\n",VERSION);
-  fprintf(stderr,"Usage:\t%s [options] src user@host:dest\nOR",RSYNC_NAME);
-  fprintf(stderr,"\t%s [options] user@host:src dest\n\n",RSYNC_NAME);
-  fprintf(stderr,"Options:\n");
-  fprintf(stderr,"-v       : increase verbosity\n");
-  fprintf(stderr,"-c       : always checksum\n");
-  fprintf(stderr,"-a       : archive mode (same as -rlptDog)\n");
-  fprintf(stderr,"-r       : recurse into directories\n");
-  fprintf(stderr,"-b       : make backups (default ~ extension)\n");
-  fprintf(stderr,"-u       : update only (don't overwrite newer files)\n");
-  fprintf(stderr,"-l       : preserve soft links\n");
-  fprintf(stderr,"-p       : preserve permissions\n");
-  fprintf(stderr,"-o       : preserve owner (root only)\n");
-  fprintf(stderr,"-g       : preserve group\n");
-  fprintf(stderr,"-D       : preserve devices (root only)\n");
-  fprintf(stderr,"-t       : preserve times\n");  
-  fprintf(stderr,"-e cmd   : specify rsh replacement\n");
+  fprintf(f,"rsync version %s Copyright Andrew Tridgell and Paul Mackerras\n\n",
+	  VERSION);
+  fprintf(f,"Usage:\t%s [options] src user@host:dest\nOR",RSYNC_NAME);
+  fprintf(f,"\t%s [options] user@host:src dest\n\n",RSYNC_NAME);
+  fprintf(f,"Options:\n");
+  fprintf(f,"-v, --verbose            increase verbosity\n");
+  fprintf(f,"-c, --checksum           always checksum\n");
+  fprintf(f,"-a, --archive            archive mode (same as -rlptDog)\n");
+  fprintf(f,"-r, --recursive          recurse into directories\n");
+  fprintf(f,"-b, --backup             make backups (default ~ extension)\n");
+  fprintf(f,"-u, --update             update only (don't overwrite newer files)\n");
+  fprintf(f,"-l, --links              preserve soft links\n");
+  fprintf(f,"-p, --perms              preserve permissions\n");
+  fprintf(f,"-o, --owner              preserve owner (root only)\n");
+  fprintf(f,"-g, --group              preserve group\n");
+  fprintf(f,"-D, --devices            preserve devices (root only)\n");
+  fprintf(f,"-t, --times              preserve times\n");  
+  fprintf(f,"-B, --block-size SIZE    checksum blocking size\n");  
+  fprintf(f,"-e, --rsh COMMAND        specify rsh replacement\n");
+  fprintf(f,"-C, --cvs-exclude        auto ignore files in the same way CVS does\n");
+  fprintf(f,"    --exclude FILE       exclude file FILE\n");
+  fprintf(f,"    --exclude-from FILE  exclude files listed in FILE\n");
+  fprintf(f,"    --suffix SUFFIX      override backup suffix\n");  
+  fprintf(f,"    --version            print version number\n");  
+
+  fprintf(f,"\n");
+  fprintf(f,"the backup suffix defaults to %s\n",BACKUP_SUFFIX);
+  fprintf(f,"the block size defaults to %d\n",BLOCK_SIZE);  
 }
 
+enum {OPT_VERSION,OPT_SUFFIX,OPT_SENDER,OPT_SERVER,OPT_EXCLUDE,OPT_EXCLUDE_FROM};
+
+static char *short_options = "oblpguDCtcahvre:B:";
+
+static struct option long_options[] = {
+  {"version",     0,     0,    OPT_VERSION},
+  {"server",      0,     0,    OPT_SERVER},
+  {"sender",      0,     0,    OPT_SENDER},
+  {"exclude",     1,     0,    OPT_EXCLUDE},
+  {"exclude-from",1,     0,    OPT_EXCLUDE_FROM},
+  {"help",        0,     0,    'h'},
+  {"cvs-exclude", 0,     0,    'C'},
+  {"archive",     0,     0,    'a'},
+  {"checksum",    0,     0,    'c'},
+  {"backup",      0,     0,    'b'},
+  {"update",      0,     0,    'u'},
+  {"verbose",     0,     0,    'v'},
+  {"recursive",   0,     0,    'r'},
+  {"devices",     0,     0,    'D'},
+  {"perms",       0,     0,    'p'},
+  {"links",       0,     0,    'l'},
+  {"owner",       0,     0,    'o'},
+  {"group",       0,     0,    'g'},
+  {"times",       0,     0,    't'},
+  {"rsh",         1,     0,    'e'},
+  {"suffix",      1,     0,    OPT_SUFFIX},
+  {"block-size",  1,     0,    'B'},
+  {0,0,0,0}};
 
 int main(int argc,char *argv[])
 {
     int i, pid, status, pid2, status2;
     int opt;
-    extern char *optarg;
-    extern int optind;
+    int option_index;
     char *shell_cmd = NULL;
     char *shell_machine = NULL;
     char *shell_path = NULL;
@@ -331,17 +386,40 @@ int main(int argc,char *argv[])
 
     starttime = time(NULL);
 
-    while ((opt=getopt(argc, argv, "oblpguDtcahvSsre:B:")) != EOF)
+    while ((opt = getopt_long(argc, argv, 
+			      short_options, long_options, &option_index)) 
+	   != -1)
       switch (opt) 
 	{
+	case OPT_VERSION:
+	  printf("rsync version %s  protocol version %d\n",
+		 VERSION,PROTOCOL_VERSION);
+	  exit(0);
+
+	case OPT_SUFFIX:
+	  backup_suffix = optarg;
+	  exit(0);
+
+	case OPT_EXCLUDE:
+	  add_exclude(optarg);
+	  break;
+
+	case OPT_EXCLUDE_FROM:
+	  add_exclude_file(optarg);
+	  break;
+
 	case 'h':
-	  usage();
+	  usage(stdout);
 	  exit(0);
 
 	case 'b':
 	  make_backups=1;
 	  break;
 
+	case 'C':
+	  cvs_exclude=1;
+	  break;
+
 	case 'u':
 	  update_only=1;
 	  break;
@@ -404,13 +482,13 @@ int main(int argc,char *argv[])
 	  }	    
 	  break;
 
-	case 's':
+	case OPT_SERVER:
 	  server = 1;
 	  break;
 
-	case 'S':
+	case OPT_SENDER:
 	  if (!server) {
-	    usage();
+	    usage(stderr);
 	    exit(1);
 	  }
 	  sender = 1;
@@ -438,6 +516,7 @@ int main(int argc,char *argv[])
       argv++;
     }
 
+
     if (server) {
       int version = read_int(STDIN_FILENO);
       if (version != PROTOCOL_VERSION) {
@@ -448,15 +527,19 @@ int main(int argc,char *argv[])
       write_int(STDOUT_FILENO,PROTOCOL_VERSION);
       write_flush(STDOUT_FILENO);
 	
-      if (sender)
+      if (sender) {
+	recv_exclude_list(STDIN_FILENO);
+	if (cvs_exclude)
+	  add_cvs_excludes();
 	do_server_sender(argc,argv);
-      else
+      } else {
 	do_server_recv(argc,argv);
+      }
       exit(0);
     }
 
     if (argc < 2) {
-      usage();
+      usage(stderr);
       exit(1);
     }
 
@@ -474,7 +557,7 @@ int main(int argc,char *argv[])
 
       p = strchr(argv[argc-1],':');
       if (!p) {
-	usage();
+	usage(stderr);
 	exit(1);
       }
 
@@ -502,7 +585,7 @@ int main(int argc,char *argv[])
     signal(SIGCHLD,SIG_IGN);
 
     if (!sender && argc != 1) {
-      usage();
+      usage(stderr);
       exit(1);
     }
 
@@ -523,6 +606,8 @@ int main(int argc,char *argv[])
 	      (int)getpid(),pid,sender,recurse);
 
     if (sender) {
+      if (cvs_exclude)
+	add_cvs_excludes();
       flist = send_file_list(f_out,recurse,argc,argv);
       if (verbose > 3) 
 	fprintf(stderr,"file list sent\n");
@@ -534,6 +619,8 @@ int main(int argc,char *argv[])
       exit(status);
     }
 
+    send_exclude_list(f_out);
+
     flist = recv_file_list(f_in);
     if (flist->count == 0) {
       exit(0);
